<?php
namespace FM\Service;

use FM\Factory\SolicitacaoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class SolicitacaoService {
    
    protected $SolicitacaoFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    
    public function __construct (SolicitacaoFactory $SolicitacaoFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager) {
        $this->SolicitacaoFactory           = $SolicitacaoFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection(); 
    }
    
    // Busca as solicitações
    public function getRequests(){
        $requestsList = $this->SolicitacaoFactory->getRequests();
        return $requestsList;
    }
    
    // Busca os tipos de solicitação
    public function getRequestTypes(){
        $typesList = $this->SolicitacaoFactory->getRequestTypes();
        return $typesList;
    }
    
   // Busca as informações de uma solicitação específica
    public function getRequestInfo($nrsolicitacao){
        $requestInfo = $this->SolicitacaoFactory->getRequestInfo($nrsolicitacao);
        return $requestInfo;
    }
    
    // Busca o histórico de uma solicitação
    public function getRequestHistoric($nrsolicitacao){
        $requestInfo = $this->SolicitacaoFactory->getRequestHistoric($nrsolicitacao);
        return $requestInfo;
    }
    
    // Busca todas as hierarquias
    public function insertRequestType($params){
        $nrtiposolicitacao  = $this->newCode->getCodeByTableName('FCM_TIPOSOLICITACAO', 20, 1, 10, true);
        
        $typeParams = array(
            'NRTIPOSOLICITACAO' => $nrtiposolicitacao,
            'NMTIPOSOLICITACAO' => $params['NMTIPOSOLICITACAO'],
            'IDPRIORIDADEPADR' => $params['IDPRIORIDADEPADR'],
            'NRORG' => 20
        );

        try {
            $this->SolicitacaoFactory->insertRequestType($typeParams);
            $response = "Inserido com sucesso";
        } catch (\Exception $e) {
            $response = $e->getMessage();
        }

        return $response;
    }
    
    // Processo de clasificação -> Alteração nas tabelas de solicitação/sol_tarefas, checklistAgend e  histórico
    public function classifyRequest($params){
        $this->connection->beginTransaction();
        try {
            $requestParams = array(
                'NRSOLICITACAO' => $params['NRSOLICITACAO'],
                'NMSOLICITACAO' => $params['NMSOLICITACAO'],
                'IDSITUACAO' => '2',
                'NRTIPOSOLICITACAO' => $params['NRTIPOSOLICITACAO'],
                'IDPRIORIDADE' => $params['IDPRIORIDADE'],
                'NRFUNCRESPONSAVEL' => $params['NRFUNCRESPONSAVEL'],
                'DSSOLICITACAO' => $params['DSSOLICITACAO']
            );
            $this->SolicitacaoFactory->updateRequest($requestParams);
            $nrChecklistAgend  = $this->newCode->getCodeByTableName('FCM_CHECKLISTAGEND', 20, 1, 10, true);
            $this->insertNovaTarefa(array(
                "NRCHECKLISTAGEND" => $nrChecklistAgend,
                "NMTAREFA" => $params['NMSOLICITACAO'],
                "LOCAL" => [$params['NRLOCALCLIENTE']],
                "CHECKLIST" => $params['NRCHECKLIST'],
                "CDOPERADOR" => $params['CDOPERADOR'],
                "DATA" => [array(
                        "DATAINICIAL" => (date('I') == 1) ? date("Y/m/d H:i", strtotime('-1 Hours')) : date('Y/m/d H:i'), 
                        "DATAFINAL" => (date('I') == 1) ? date("Y/m/d H:i", strtotime('-1 Hours +30 Minutes')) : date('Y/m/d H:i', strtotime('+30 Minutes')),
                        "NRFUNCIONARIO" => $params["NRFUNCRESPONSAVEL"]
                    )]
            ));
            $nrSolicitacaoTarefas  = $this->newCode->getCodeByTableName('FCM_SOLICITACAOTAREFAS', 20, 1, 10, true);    
            $taskRequestParams = array(
                'NRSOLICITACAOTAREFAS' => $nrSolicitacaoTarefas,
                'NRSOLICITACAO' => $params['NRSOLICITACAO'],
                'NRCHECKLISTAGEND' => $nrChecklistAgend,
                'NRORG' => 20
            );
            $this->SolicitacaoFactory->createTaskRequest($taskRequestParams); // Cria uma relação de tarefa com solicitação
            
            $nrSolicitacaoHistorico  = $this->newCode->getCodeByTableName('FCM_SOLICITACAOHISTORICO', 20, 1, 10, true);    
            $historicParams = array(
                'NRSOLICITACAOHISTORICO' => $nrSolicitacaoHistorico,
                'NRSOLICITACAO' => $params['NRSOLICITACAO'],
                'NMSOLICITACAO' => $params['NMSOLICITACAO'],
                'DTHISTSOLICITACAO' => $params['DTHISTSOLICITACAO'],
                'IDSITUACAO' => '2',
                'IDTIPOHISTORICO' => '2',
                'NRFUNCSOLICITANTE' => $params['NRFUNCRESPONSAVEL'],
                'DSSOLICITACAO' => $params['DSSOLICITACAO'],
                'NRORG' => 20,
            );
            $this->SolicitacaoFactory->insertHistoricData($historicParams);
            
            $this->connection->commit();
            $response = "Classificado com sucesso";
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response = $e->getMessage();
        }

        return $response;
    }
    
    public function insertNovaTarefa($param){
        $this->connection->beginTransaction();
        try {
            $NRORG = 20;
            forEach($param['DATA'] as $dados){
                
                forEach($param['LOCAL'] as $local){
                    $nrchecklistagend = $param['NRCHECKLISTAGEND'];
                    $params = array(
                        "NRCHECKLISTAGEND" => $nrchecklistagend,
                        "NMCHECKLISTAGEND" => $param['NMTAREFA'],
                        "NRLOCALCLIENTE"  => $local,
                        "NRFUNCIONARIO" => $dados['NRFUNCIONARIO'],
                        "IDSITUACAOAGEND" => 1,
                        "DTHORAAGEND" => $dados['DATAINICIAL'],
                        "DTHRFINALAGEND" => $dados['DATAFINAL'],
                        "OPERADOR" => $param['CDOPERADOR'],
                        "NRCHECKLIST"  => $param['CHECKLIST'],
                    );
                     $this->SolicitacaoFactory->insertNovaTarefa($params);
                    
                    if($param['CHECKLIST']){
                        $tarefas = $this->SolicitacaoFactory->getTarefaChk(array("NRCHECKLIST" =>  $param['CHECKLIST']));
                    }else{
                        $tarefas["NRTAREFACHK"] = null;
                    }
                    
                    forEach($tarefas as $dadosTarefa){
                        $nrtarefaagend = $this->newCode->getCodeByTableName("FCM_TAREFAAGEND", $NRORG, 1, 10, true);
                        $params = array(
                            "NRTAREFAAGEND" => $nrtarefaagend,
                            "NRCHECKLISTAGEND" => $nrchecklistagend,
                            "NRTAREFACHK" => $dadosTarefa["NRTAREFACHK"],
                            "NRFUNCIONARIO" => $dados['NRFUNCIONARIO'],
                            "DTHORAAGEND" => $dados['DATAINICIAL'], 
                            "DSOBSTAREFAAGEND" => null,
                            "OPERADOR" => $param['CDOPERADOR']
                        );
                        $operadores = $this->SolicitacaoFactory->insertNovaTarefa2($params);
                        
                    };
                }
            }
            $this->connection->commit();
            $response = "OK";
        }
        catch(\Exception $e) {
             $this->connection->rollback();
             $response = utf8_decode($e->getErrorCode());
        }
        return $response;
    }
    
     // Edita o tipo de solicitação
    public function updateRequestType($params){
        $typeParams = array(
            'NRTIPOSOLICITACAO' => $params['NRTIPOSOLICITACAO'],
            'NMTIPOSOLICITACAO' => $params['NMTIPOSOLICITACAO'],
            'IDPRIORIDADEPADR' => $params['IDPRIORIDADEPADR']
        );
        
        try {
            $this->SolicitacaoFactory->updateRequestType($typeParams);
            $response = "Editado com sucesso";
        } catch (\Exception $e) {
            $response = $e->getMessage();
        }

        return $response;
    }
    
    public function getLocalRequest($params) {
        $locals = $this->SolicitacaoFactory->getLocalRequest($params);
        return $locals;
    }
    
    public function newTarefa($params) {
        return $this->SolicitacaoFactory->newTarefa($params);
    }

    public function newTarefaChk($params) {
        return $this->SolicitacaoFactory->newTarefaChk($params);
    }
    
    public function setNewRequest($parametros) {

        // debugApi([$parametros]);
        $this->connection->beginTransaction();

         try {
                $NRORG = 20;
                $NRSOLICITACAO = $this->newCode->getCodeByTableName("FCM_SOLICITACAO", $NRORG, 1, 10, true);
               //  debugApi(['oi']);
                $params = array(
                    "NRSOLICITACAO"  => $NRSOLICITACAO,
                    "NMSOLICITACAO"  => $parametros['NMSOLICITACAO'],
                    "IDSITUACAO"     => $parametros['IDSITUACAO'],
                    "NRLOCALCLIENTE" => $parametros['NRLOCALCLIENTE'],
                    "NRFUNCABERTURA" => $parametros['NRFUNCABERTURA'],
                    "DSSOLICITACAO"  => $parametros['DSSOLICITACAO'],
                    "NRORG"          => $NRORG
                );
                // debugApi([$params]);
                $this->SolicitacaoFactory->insertNewRequest($params);

                $NRSOLICITACAOHISTORICO = $this->newCode->getCodeByTableName("FCM_SOLICITACAOHISTORICO", $NRORG, 1, 10, true);

                $params = array(
                    "NRSOLICITACAOHISTORICO" => $NRSOLICITACAOHISTORICO,
                    "NRSOLICITACAO"          => $NRSOLICITACAO,
                    "IDSITUACAO"             => 1,
                    "IDTIPOHISTORICO"        => 1,
                    "NRFUNCSOLICITANTE"      => $parametros['NRFUNCABERTURA'],
                    "DSSOLICITACAO"          => $parametros['DSSOLICITACAO'],
                    "IDSATISFACAOSOLICITA"   => null,
                    "NRORG"                  => $NRORG
                );
                // debugApi([$params, '1']);

                $this->SolicitacaoFactory->insertRequestHistory($params);

                // debugApi([$params]);

            $this->connection->commit();
            $response = "OK";

        } catch(\Exception $e) {

             $this->connection->rollback();
             $response = ($e->getMessage());
        }
        return $response;
    }

    public function getCardsRequests($parametro, $parametroIds) {

        $param = '';

        if ($parametro['selectName'] != '' && $parametro['selectName'] != null) {
            $param = $param." AND S.NRSOLICITACAO = ".$parametro['selectName'];
        }

        if ($parametro['selectLocal'] != '' && $parametro['selectLocal'] != null) {
            $param = $param." AND S.NRLOCALCLIENTE = ".$parametro['selectLocal'];
        }

        if ($parametro['selectSituation'] != '' && $parametro['selectSituation'] != null) {
            $param = $param." AND S.IDSITUACAO = ".$parametro['selectSituation'];
        }

        if ($parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')'] != '' && $parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')'] != null && strlen($parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')']) == 21) {
            $param = $param." AND TO_CHAR(S.DTSOLICITACAO,'DD/MM/YYYY') = ".$parametro['TO_CHAR(S.DTSOLICITACAO,\'DD/MM/YYYY\')'];
        }

        $usersIds = implode(',',$parametroIds); // Transforma o array em uma string com os ids

        $cards = $this->SolicitacaoFactory->getCardsRequests($param, $usersIds);
        return $cards;
    }

    public function showRequest($params) {

        $parametros = array(
            "NRSOLICITACAO"    => $params['NRSOLICITACAO'],
        );

        $details = $this->SolicitacaoFactory->showRequest($parametros);
        return $details;
    }

    public function getSituationRequest() {

        $situation = $this->SolicitacaoFactory->getSituationRequest();
        return $situation;
    }

    public function getNameRequest() {

        $nameRequest = $this->SolicitacaoFactory->getNameRequest();
        return $nameRequest;
    }

    public function evaluateRequest($parametros) {
        $this->connection->beginTransaction();

         try {
            $NRORG = 20;
            $params = array(
                "NRSOLICITACAO"        => $parametros['NRSOLICITACAO'],
                "IDSITUACAO"           => ($parametros['CONDITION']) ? 4 : 1,
                "DSSOLICITACAO"        => $parametros['DSSOLICITACAO']
            );
            $this->SolicitacaoFactory->upRequest($params);
            
            $NRSOLICITACAOHISTORICO = $this->newCode->getCodeByTableName("FCM_SOLICITACAOHISTORICO", $NRORG, 1, 10, true);

            $params = array(
                    "NRSOLICITACAOHISTORICO" => $NRSOLICITACAOHISTORICO,
                    "NRSOLICITACAO"          => $parametros['NRSOLICITACAO'],
                    "IDSITUACAO"             => ($parametros['CONDITION']) ? 4 : 1,
                    "IDTIPOHISTORICO"        => 4,
                    "NRFUNCSOLICITANTE"      => $parametros['NRFUNCABERTURA'],
                    "DSSOLICITACAO"          => $parametros['DSSOLICITACAO'],
                    "IDSATISFACAOSOLICITA"   => $parametros['IDSATISFACAOSOLICITA'],
                    "NRORG"                  => $NRORG
                );
            $this->SolicitacaoFactory->insertRequestHistory($params);

            $this->connection->commit();
            $response = "OK";

        } catch(\Exception $e) {

             $this->connection->rollback();
             $response = ($e->getMessage());
        }
        return $response;
    }
}
?>